indexing
	description: "High Score Table for TRIO Game"
	author: "Dr. Thomas Tensi"
	date: "$Date$"
	revision: "$Revision$"

class
	HIGH_SCORE_TABLE

inherit
	WEL_REGISTRY

	EXECUTION_ENVIRONMENT
		rename
			get as environment_get,
			put as environment_put
		end


create
	make

feature -- Initialization

	make is
			-- read from registry when available or clear it totally
		local
			d : DATE
			entry : TUPLE[STRING,DATE,INTEGER]
			field_list : LIST[STRING]
			i : INTEGER
			high_score : STRING
			high_score_registry_value : WEL_REGISTRY_KEY_VALUE
			high_score_row : STRING
			row_list : LIST[STRING]
		do
			create {ARRAYED_LIST[TUPLE[STRING,DATE,INTEGER]]}list
					.make_filled(count)

			clear_list
			high_score_registry_value := open_key_value(high_score_registry_path,
														high_score_registry_variable)

			if high_score_registry_value /= void then
				-- parse high_score
				high_score := high_score_registry_value.string_value.as_string_8
				row_list := high_score.split(score_row_separator)

				from
					i := 1
				until
					i > row_list.count or i > count
				loop
					high_score_row := row_list[i]
					field_list := high_score_row.split(entry_field_separator)

					if field_list.count = 3 then
						create d.make_by_days((field_list[2]).to_integer)
						entry := [field_list[1], d, (field_list[3]).to_integer]
						list[i] := entry
					end

					i := i + 1
				end

				sort_list
			end
		end


feature -- Access

	count : INTEGER is 10
		-- number of slots in high score table

	list : LIST[TUPLE[STRING, DATE, INTEGER]]
		-- the persons and their score in the table sorted in descending order


feature -- Element change

	put (score : INTEGER) is
			-- adds `score' to high score table if good enough for current user
		local
			current_date : DATE
			entry : TUPLE[STRING,DATE,INTEGER]
			user_name : STRING
		do
			user_name := environment_get(user_name_variable)

			if user_name = void then
				user_name := "???"
			end

			create current_date.make_now
			entry := [user_name, current_date, score]

			list.extend(entry)
			sort_list
			list.go_i_th(list.count)
			list.remove

			if score >= (list[count]).integer_item(3) then
				store_list
			end

		ensure
			has_correct_count : list.count = count
			in_descending_order : is_correctly_sorted
		end

feature -- Conversion

	to_string : STRING is
			-- returns a multi-line representation of high score list
		local
			current_date : DATE
			entry : TUPLE[STRING,DATE,INTEGER]
			i  : INTEGER
			st : STRING
		do
			create st.make_empty

			from
				i := 1
			until
				i > count
			loop
				entry := list[i]
				st.append(entry.item(1).out)
				st.append_character('%T')
				current_date ?= entry.item(2)
				st.append(current_date.formatted_out("yyyy-[0]mm-[0]dd"))
				st.append_character('%T')
				st.append_integer(entry.integer_item(3))
				st.append_character('%N')

				i := i + 1
			end

			Result := st
		end



feature {NONE} -- Implementation

	entry_field_separator : CHARACTER is ''
		-- character used for separating name and score in score entry

	score_row_separator : CHARACTER is '/'
		-- character used for separating the rows in registry entry

	high_score_registry_path : STRING_8 is
		"HKEY_LOCAL_MACHINE\Software\Tensi Software\Trio"
		-- path in registry to access high score

	high_score_registry_variable : STRING_8 is "high_score"
		-- variable in registry taking high score table

	user_name_variable : STRING_8 is "USERNAME"
		-- environment variable name for login name

	-- --------------------

	clear_list is
			-- reset high score list to zero values
		local
			default_date : DATE
			i : INTEGER
			st : STRING_8
		do
			create default_date.make(2000, 1, 1)

			from
				i := 1
			until
				i > count
			loop
				st := "---"
				list[i] := [st.to_string_32, default_date, 0]
				i := i + 1
			end
		end

	-- --------------------

	is_correctly_sorted : BOOLEAN is
			-- tell whether list is sorted in score descending order
		local
			current_value : INTEGER
			i : INTEGER
			is_okay : BOOLEAN
			previous_value : INTEGER
		do
			previous_value := -1
			is_okay := true

			from
				i := list.count
			until
				i < 1 or not is_okay
			loop
				current_value := (list[i]).integer_item(3)
				is_okay := (current_value >= previous_value)
				i := i - 1
			end

			Result := is_okay
		end

	-- --------------------

	sort_list is
			-- sort list in score descending order
		local
			i : INTEGER
			j : INTEGER
			high_score_entry : TUPLE[STRING,DATE,INTEGER]
			v1 : INTEGER
			v2 : INTEGER
		do
			-- bubble sort
			from
				i := list.count - 1
			until
				i < 1
			loop
				from
					j := 1
				until
					j > i
				loop
					v1 := list.i_th(j).integer_item(3)
					v2 := list.i_th(j+1).integer_item(3)

					if v1 < v2 then
						-- swap
						high_score_entry := list[j]
						list[j] := list[j+1]
						list[j+1] := high_score_entry
					end

					j := j + 1
				end

				i := i - 1
			end
		end

	-- --------------------

	store_list is
			-- store list in windows registry
		local
			current_date : DATE
			entry : TUPLE[STRING, DATE, INTEGER]
			i : INTEGER
			registry_type : WEL_REGISTRY_KEY_VALUE_TYPE
			st : STRING
			value : WEL_REGISTRY_KEY_VALUE
		do
			create registry_type
			create st.make_empty

			from
				i := 1
			until
				i > count
			loop
				entry := list[i]
				st.append(entry.item(1).out)
				st.append_character(entry_field_separator)
				current_date ?= entry.item(2)
				st.append_integer(current_date.days)
				st.append_character(entry_field_separator)
				st.append_integer(entry.integer_item(3))
				st.append_character(score_row_separator)
				i := i + 1
			end

			create value.make(registry_type.reg_sz, st)
			save_key_value(high_score_registry_path, high_score_registry_variable,
			               value)
		end

end
